#!perl

package This;
use warnings;
use strict;
use Test::More;

use XML::LibXML;

use DateTime;

my $parser = XML::LibXML->new;
my $rngschema = XML::LibXML::RelaxNG->new( location => "/usr/local/nagios/share/xml/opsview.rng" );

use LWP::UserAgent;
my $ua = LWP::UserAgent->new;

sub test {
    my ( $class, $req ) = @_;
    ok( my $res = $ua->request($req), "Request" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $res->content_type, "text/xml", "XML returned" );
    return $res;
}

package main;
use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";

use Opsview::Web::Test;

use Test::More tests => 26;

use HTTP::Request::Common;
use HTTP::Request;
use POSIX qw( strftime );

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";
use Opsview;
use Opsview::Config;
use Opsview::Rancidvendor;

use Sys::Hostname;
my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# Need to create as this doesn't exist in default db
Opsview::Rancidvendor->create( { name => "Cisco", rancid_name => "cisco" } );

# Create a new host with rancid information
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			<icon><name>LOGO - Opsview</name></icon>
			<use_rancid>1</use_rancid>
			<rancid_username>user1</rancid_username>
			<rancid_password>pass123</rancid_password>
			<rancid_vendor><name>Cisco</name></rancid_vendor>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

my $host = Opsview::Host->search( name => "testviaxml" )->first;
is( $host->use_rancid,          1,         "use_rancid set" );
is( $host->rancid_username,     "user1",   "rancid_username set" );
is( $host->rancid_password,     "pass123", "rancid_password set" );
is( $host->rancid_vendor->name, "Cisco",   "rancid_vendor set" );

# Good create via clone
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
                        <authentication><username>admin</username><password>initial</password></authentication>
                        <host action="create">
                        <clone><name>testviaxml</name></clone>
                        <name>cloned</name>
                        <use_rancid>0</use_rancid>
                        </host>
                        </opsview>
                        },
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

$host = Opsview::Host->search( name => "cloned" )->first;
is( $host->use_rancid,          0,         "use_rancid set" );
is( $host->rancid_username,     "user1",   "rancid_username set" );
is( $host->rancid_password,     "pass123", "rancid_password set" );
is( $host->rancid_vendor->name, "Cisco",   "rancid_vendor set" );

{
    local $TODO = "cannot yet create multiple clones";

    # Good double create via clone
    {
        my $res = This->test(
            HTTP::Request->new(
                "PUT" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<host action="create">
				<clone><name>testviaxml</name></clone>
				<name>cloned_2</name>
				<use_rancid>0</use_rancid>
				</host>
				<host action="create">
				<clone><name>testviaxml</name></clone>
				<name>cloned_3</name>
				<use_rancid>0</use_rancid>
				</host>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Success" );
        is( $res->code, 200, "Status 200" );
        like( $res->content, "/OK/s", "Expected XML" );
    }

    #my $host1 = Opsview::Host->search( name => "cloned_2" )->first;
    #is( $host1->use_rancid, 0, "use_rancid set");
    #is( $host1->rancid_username, "user1", "rancid_username set");
    #is( $host1->rancid_password, "pass123", "rancid_password set");
    #is( $host1->rancid_vendor->name, "Cisco", "rancid_vendor set");
    #my $host2 = Opsview::Host->search( name => "cloned_3" )->first;
    #is( $host2->use_rancid, 0, "use_rancid set");
    #is( $host2->rancid_username, "user1", "rancid_username set");
    #is( $host2->rancid_password, "pass123", "rancid_password set");
    #is( $host2->rancid_vendor->name, "Cisco", "rancid_vendor set");

}
