#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";

use Opsview::Web::Test;

use Test::More tests => 28;

use HTTP::Request::Common;
use HTTP::Request;
use POSIX qw( strftime );

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";

use Sys::Hostname;

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

use LWP::UserAgent;
my $ua = LWP::UserAgent->new;

# No username/password
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request without username/password" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Username but no password
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "Content-Type" => "text/xml" );
    $r->header( "X-Username"   => "admin" );
    ok( my $res = $ua->request($r), "Request with username but no password" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Password but no username
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "Content-Type" => "text/xml" );
    $r->header( "X-Password"   => "initial" );
    ok( my $res = $ua->request($r), "Request with password, but no username" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Credentials via URL: username, no password
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?login_username=admin" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request with username via URL params" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Credentials via URL: password, no username
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?login_password=initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request with password via URL params" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Credentials via URL: ok
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?login_username=admin&login_password=initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request logged in via URL" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    unlike( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Credentials via URL: check for some weird error if an arrayref is returned
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?login_username=admin&login_password=initial&login_username=admin&login_password=initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request logged in via URL" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    unlike( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}
