#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More;
use Test::Deep;

plan tests => 8;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize::Catalyst->new;

log_in( $mech, 'admin', 'initial' );

is( test_all_images( $mech, "/admin/hostgroup" ),       1, "Got all images for hostgroup page" );
is( test_all_images( $mech, "/admin/hostgroup/graph" ), 1, "Got all images for hostgroup graph page" );
is( test_all_images( $mech, "/admin/servicechecks" ),   1, "Got all images for service checks page" );

# Ignore hosts for now, because dummy data has logo types which don't exist yet
#is( test_all_images( $mech, "/admin/host" ), 1, "Got all images" );
#$mech->get_ok("/admin/host");
#$mech->page_links_ok( "All page links okay" );

sub test_all_images {
    my ( $mech, $url ) = @_;
    $mech->get_ok($url);
    my @images = $mech->find_all_images;
    my $fails  = 0;
    foreach my $image (@images) {
        my $url = $image->url;
        if ( $url eq "" ) {
            warn("No image url found");
            $fails++;
            next;
        }
        $mech->get( $image->url );
        if ( !$mech->success ) {
            $fails++;
            warn( "Got failure for url: " . $image->url );
        }
    }
    return ( $fails == 0 );
}
