#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/perl/lib", "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use Test::LongString;

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize->new;
my $configurehosts = Test::WWW::Mechanize->new;

log_in( $mech, 'admin', 'initial', "http://localhost:3000" );
log_in( $configurehosts, 'somehosts', 'somehosts', "http://localhost:3000" );

$mech->get_ok("/admin/host/edit/1");
$mech->content_contains("This is a master monitoring server");
$mech->content_lacks("Parents:");
$mech->content_lacks("Monitored By:");

$mech->get_ok("/admin/host/edit/5");
$mech->content_contains("This is a slave monitoring server");
$mech->content_contains("Parents:");
$mech->content_lacks("Monitored By:");

$mech->get_ok("/admin/host/edit/7");
$mech->content_contains("Parents:");
$mech->content_contains("Monitored By:");
$mech->form_name("main_form");
is( $mech->value( "hostgroup" ), 4, "Host group set to Leaf" );
is( $mech->value( "flap_detection_enabled"), 1, "Flap detection is set" );
is( $mech->value( "enable_snmp"), undef, "SNMP not enabled" );
$mech->field("hostgroup", "singlehost" );
$mech->untick("flap_detection_enabled", 1);
$mech->tick("enable_snmp", 1);
$mech->submit_form;

$mech->content_unlike(qr/error in the Opsview/);

$mech->get_ok("/admin/host/edit/7");
$mech->form_name("main_form");
is( $mech->value( "hostgroup" ), 7, "Host group changed to singlehost" );
is( $mech->value( "flap_detection_enabled"), undef, "Flap detection is changed to 0" );
is( $mech->value( "enable_snmp"), 1, "SNMP is enabled" );
$mech->field("hostgroup_new", "newlymintedhostgroup" );
$mech->untick("enable_snmp", 1);
$mech->submit_form;

$mech->get_ok("/admin/host/edit/7");
$mech->form_name("main_form");
is( $mech->value( "hostgroup" ), 11, "Host group added" );
is( $mech->value( "enable_snmp"), undef, "SNMP is disabled" );
$mech->content_contains( "newlymintedhostgroup", "Found in list too" );

# This test is a bit silly because hostgroup_new will not be visible
# in the UI and Mechanize will not allow setting a value. But have tested 
# by allowing the textfield to be available
# Would be better to do a different test that submitted some values to the
# webapp appropriately
$configurehosts->get_ok("/admin/host/edit/4");
$configurehosts->form_name( "main_form" );
is( $configurehosts->value( "hostgroup" ), 5, "This is leaf2" );
$configurehosts->content_lacks("hostgroup_new", "No hostgroup_new field" );
# Test below fails on hudson, though not sure why. The above basically does the same.
#eval { 
#  $configurehosts->field("hostgroup_new", "newlymintedhostgroup2" );
#};
#like( $@, qr/^No such field 'hostgroup_new' at/, "No hostgroup_new field correctly" );
$configurehosts->submit_form;
is( $configurehosts->status, 200, "Saved" );

$configurehosts->get_ok("/admin/host/edit/4" );
$configurehosts->form_name ("main_form" );
is( $configurehosts->value( "hostgroup" ), 5, "Still leaf2 because hostgroup_new ignored" );
$configurehosts->content_lacks( "newlymintedhostgroup2", "And not created too" );

$mech->get_ok("/admin/host");
$mech->form_name("search_list");
$mech->field("q", "ops");
$mech->submit_form;

$mech->title_is("Opsview > Host > List");
$mech->form_name("search_list");
is( $mech->value("q"), "ops", "search field correct" );

my $rows;
$rows = grep /Edit /, split( "\n", $mech->content );
is( $rows, 7, "Got correct number of search rows" );

$mech->get_ok("/admin/host/edit/4");
$mech->form_name("search_list");
$mech->field("q", "ops");
$mech->submit_form;

$mech->title_is("Opsview > Host > List" );
$mech->form_name("search_list");
is($mech->value("q"), "ops", "search field populated" );

$rows = grep /Edit /, split("\n", $mech->content );
is( $rows, 7, "Still get 7 rows" );

$configurehosts->get_ok("/admin/host/delete/4", "Deleted host" );
$configurehosts->content_contains('http-equiv="refresh"', "Response is a redirect" );

$mech->get_ok("/admin/host/delete/2", "Deleted host with MRTG" );
$mech->content_contains('http-equiv="refresh"', "Response is a redirect" );

$mech->get_ok("/admin/host/new");
$mech->content_contains('<a href="#tab-host">Host</a>', "Got tab for 1st host" );
$mech->content_contains('<li class="empty" title="These tabs are available on the next screen">Notifications</li>', "Blank tabs for others");

$mech->form_name("main_form");
$mech->field("name", "testme");
$mech->field("ip", "testme.example.com");
$mech->submit_form;

is( $mech->status, 200, "Submitted okay" );
is( $mech->uri, "http://localhost:3000/admin/host/save_new", "Got save_new post");
is( $mech->content, qq{<html>
  <head>
    <meta http-equiv="refresh" content="0; URL=http://localhost:3000/admin/host/edit/25?from=http%3A%2F%2Flocalhost%3A3000%2Fadmin%2Fhost%2Fdelete%2F2&tab=2" />
  </head>
</html>}, "Got refresh to 2nd tab" );

$mech->get_ok("/admin/host/edit/25", "Get newly created host");
$mech->form_name("main_form");
is( $mech->value("name"), "testme", "Got host name" );
is( $mech->value("ip"), "testme.example.com", "Got ip address" );

$mech->get_ok("/admin/host/edit/1?from=default");
$mech->form_name("main_form");
$mech->submit_form;
is( $mech->status, 200, "Submitted okay" );
is( $mech->uri, "http://localhost:3000/admin/host/save/1", "Got save post");
is( $mech->content, qq{<html>
  <head>
    <meta http-equiv="refresh" content="0; URL=http://localhost:3000/admin/host" />
  </head>
</html>}, "Got refresh to back to list page due to default keyword" );



$mech->get_ok("/admin/hypergraph/graph_data");
open F, ">", "$Bin/var/hypergraph.generated";
print F $mech->content;
close F;
my $expected;
open F, "$Bin/var/hypergraph.expected";
{ $/ = undef; $expected = <F>; }
close F;
is_string_nows( $mech->content, $expected, "Got hypergraph data" ) || diag("See $Bin/var/hypergraph.generated vs $Bin/var/hypergraph.expected");

$mech->get_ok("/admin/host/list");
$mech->get_ok("/admin/host/clone/7");
$mech->form_name("main_form");
$mech->field("name", "ciscoclone");
$mech->submit_form;

is($mech->status, 200, "Submitted okay");
is( $mech->uri, "http://localhost:3000/admin/host/save_new", "Got save_new post");
is( $mech->content, qq{<html>
  <head>
    <meta http-equiv="refresh" content="0; URL=http://localhost:3000/admin/host/list" />
  </head>
</html>}, "Got back to list page" );


$mech->get_ok("/admin/host/new");
$mech->form_name("main_form");
$mech->field("name", "abadhost");
$mech->field("hostgroup_new", "invalid!chars");
$mech->submit_form;

is( $mech->status, 500, "Submission should fail due to exception" );
$mech->content_contains( "name: Invalid", "Error due to host group with invalid name" ) || diag($mech->content);


note("Testing exceptions changes - have to manually do ticking");
$mech->get_ok("/admin/host/edit/6"); # resolved_services
$mech->form_name("main_form");
is( $mech->value("event_handler_1_add"), "done-for-afs" );
$mech->field("event_handler_1_add", "changed event handler via testing");
is( $mech->value("timedoverride_45_time"), 1 );
$mech->select("timedoverride_45_time", 3);
is( $mech->value("timedoverride_45_cmd"), "--timed");
$mech->field("timedoverride_45_cmd", "changed timed exception via testing" );
is( $mech->value("exception_44_add"), "--exception --and" );
$mech->field("exception_44_add", "changed exception via testing" );
$mech->submit_form;

$mech->get_ok("/admin/host/edit/6");
$mech->form_name("main_form");
is( $mech->value("event_handler_1_add"), "changed event handler via testing" );
is( $mech->value("timedoverride_45_time"), 3 );
is( $mech->value("timedoverride_45_cmd"), "changed timed exception via testing" );
is( $mech->value("exception_44_add"), "changed exception via testing" );

note("Testing list of services expanded in monitors tab");
$mech->get_ok("/admin/servicegroup/services/2");
$rows = grep m%</tr>%, split( "\n", $mech->content );
is( $rows, 6, "Found list of services" );


1;
