#!/usr/bin/perl
# Tests for navigating around Opsview

use Test::More qw(no_plan);

use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/../etc";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";
use strict;
use Opsview;
use Opsview::Config;
use Test::WWW::Mechanize;
use lib "$Bin/lib";
use Opsview::TestUtils;

my $uri      = "http://localhost:3000";

my $mech = Test::WWW::Mechanize->new( stack_depth => 1 );

log_in( $mech, 'admin', 'initial', $uri );

$mech->get_ok("/");
$mech->title_is("Opsview > Host Group Summary > Opsview", "Got title" );
$mech->content_contains("UK2", "Got front page" );
$mech->content_contains('<div id="infobox">', "Got breadcrumbs infobox" );
$mech->content_lacks("Leaf", "No Leaf hostgroup listed" );

$mech->follow_link( text => "Unhandled - Hierarchy" );
$mech->content_contains( "UK2", "Have traversed into alert view" );

$mech->back;

$mech->follow_link( text => "UK" );
$mech->content_contains("Leaf", "... but now has Leaf hostgroup listed" );

$mech->follow_link( text => "Leaf" );
$mech->content_contains('<li><a href="/status/hostgroup?parentid=3">UK</a> </li>', "Has got a breadcrumb trail");

$mech->follow_link( text => "doesnt_exist_1" );
$mech->content_contains('<li><a href="/status/hostgroup?parentid=3">UK</a> </li>', "Still got a breadcrumb trail");
$mech->content_contains('<li><a href="/status/hostgroup?parentid=4">Leaf</a> </li>', "Got Leaf in breadcrumb" );
$mech->content_contains('<li> <a href="/status/service?host=doesnt_exist_1">doesnt_exist_1</a> </li></ul></div>', "Got host in breadcrumb too" );

$mech->follow_link( url_regex => qr%^/graph% );
$mech->content_contains('<li><a href="/status/hostgroup?parentid=3">UK</a> </li>', "Same breadcrumb trail...");
$mech->content_contains('<li><a href="/status/hostgroup?parentid=4">Leaf</a> </li>', "Got Leaf in breadcrumb" );
$mech->content_contains('<li> <a href="/status/service?host=doesnt_exist_1">doesnt_exist_1</a> </li></ul></div>', "Got host in breadcrumb too" );

